@@ -15,66 +15,83 @@ Graphics::Framebuffer - A Simple Framebuffer Graphics Library
  $fb->plot({'x' => 28, 'y' => 79,'pixel_size' => 1});
  $fb->drawto({'x' => 405,'y' => 681,'pixel_size' => 1});
  $fb->circle({'x' => 200, 'y' => 200, 'radius' => 100, 'filled' => 1});
+ $fb->polygon({'coords' => [20,20,  53,3,  233,620], 'pixel_size' => 5});
+ $fb->box({'x' => 95, 'y' => 100, 'xx' => 400, 'yy' => 600, 'filled' => 1});
 
  $fb->close_screen();
 
 =head1 DESCRIPTION
 
-A (mostly) Pure Perl graphics library for exclusive use in a console framebuffer
-environment.  It is written for simplicity without the need for complex API's
-and drivers.
+A (mostly) Pure Perl graphics library for exclusive use in a console
+framebuffer environment.  It is written for simplicity, without the need for
+complex API's and drivers with "surfaces" and such.
 
 Back in the old days, computers drew graphics this way, and it was simple and
 easy to do.  I was writing a console based media playing program, and was not
 satisfied with the limited abilities offered by the Curses library, and I did
 not want the overhead of the X environment to get in the way.  My intention
 was to create a mobile media server.  In case you are wondering, that project
-has been quite successful, and I am still making improvements to it.
+has been quite successful, and I am still making improvements to it.  I may
+even include it in the "examples" directory on future versions.
 
 There are places where Pure Perl just won't cut it.  So I use the Imager
 library to take up the slack.  It's just used to load and save images, and
 draw TrueType text.
 
 I cannot guarantee this will work on your video card, but I have successfully
-tested it on NVidia GeForce, AMD Radeon, Matrox,  and VirtualBox displays.
-However, you MUST remember, your video driver MUST be framebuffer based.  The
-proprietary Nvidia and AMD drivers will NOT work with this module.  You must use
-the open source video drivers, such as Nouveau, to be able to use this library.
-Also, it is not going to work from within X, so don't even try it.  This is a
-console only graphics library.
+tested it on NVidia GeForce, AMD Radeon, Matrox, Raspberry PI, and VirtualBox
+displays.  However, you MUST remember, your video driver MUST be framebuffer
+based.  The proprietary Nvidia and AMD drivers will NOT work with this module.
+You must use the open source video drivers, such as Nouveau, to be able to use
+this library.  Also, it is not going to work from within X, so don't even try
+it, at will either crash, or make a mess.  This is a console only graphics
+library.
 
-I highly suggest you use 32 bit mode and avoid 16 bit, as it has been a long time
-since I tested it on a 16 bit graphics mode.
+I have not been able to get it to work on the Odroid XU3.  Apparently its
+video driver is not a true framebuffer.  Which is sad, as a multithreaded
+program on its 8 cores would be awesome.
+
+I highly suggest you use 32 bit mode and avoid 16 bit, as it has been a long
+time since I tested it on a 16 bit graphics mode.
 
 NOTE:
 
-If a framebuffer is not available, the module will go into emulation
-mode and open a pseudo-screen in the object's hash variable 'SCREEN'
+If a framebuffer is not available, the module will go into emulation mode and
+open a pseudo-screen in the object's hash variable 'SCREEN'
 
-You can write this to a file, whatever.  It defaults to a 640x480x32
-graphics 'buffer'.  However, you can change that by passing parameters
-to the 'new' method.
+You can write this to a file, whatever.  It defaults to a 640x480x32 graphics
+'buffer'.  However, you can change that by passing parameters to the 'new'
+method.
 
-You will not be able to see the output directly when in emulation
-mode.  I mainly created this mode so that you could install this
-module and test code you may be writing to be used on other devices
-that have accessible framebuffer devices.
+You will not be able to see the output directly when in emulation mode.  I
+mainly created this mode so that you could install this module (on systems
+without a framebuffer) and test code you may be writing to be used on other
+devices that have accessible framebuffer devices.
 
 =cut
 
 use strict;
-no strict 'vars';
+no strict 'vars'; # We have to map a variable as the screen.  So strict is
+                  # going to whine about what we do with it.
+use constant {
+    RGB => 1,
+    BGR => 0
+};
 use 5.010;
-use Switch;    # Yes, a touch of new Perl.
-use Math::Trig qw(:pi);
-use Sys::Mmap;    # Absolutely necessary to map the screen to a string.
-use Imager;       # This is used for TrueType font printing.
+# use Carp;
+use Switch;             # Yes, a touch of new Perl. Switch is so much nicer
+                        # a ton of if-else's
+use Math::Trig qw(:pi); # Yes, yummy PI
+use Sys::Mmap;          # Absolutely necessary to map the screen to a string.
+                        # Without this, I doubt this module would exist.
+use Imager;             # This is used for TrueType font printing, and where
+                        # 'Pure Perl' goes up in smoke.
 
 BEGIN {
     require Exporter;
 
     # set the version for version checking
-    our $VERSION   = 4.10;
+    our $VERSION   = 4.11;
     our @ISA       = qw(Exporter);
     our @EXPORT    = qw();
     our @EXPORT_OK = qw();
@@ -105,6 +122,14 @@ my $fb = Graphics::Framebuffer->new(parameter => value);
 
  Framebuffer device name. Default is /dev/fb0
 
+=item B< COLOR_ORDER >
+
+Sets the coloring logic to use the proper color ordering.
+Currently, the only accepted values are either 1 for RGB or
+0 for BGR.
+
+The default is 0 for BGR
+
 =item B< VXRES (Emulation Mode Only!  Otherwise ignored.)>
 
  Width of the emulation framebuffer.  Default is 640.
@@ -182,14 +207,20 @@ sub new {
         'XOFFSET'                  => 0,
         'YOFFSET'                  => 0,
         'FB_DEVICE'                => '/dev/fb0',
+        'COLOR_ORDER'              => BGR,   # Either BGR or RGB
         @_
     };
-    if (open($self->{'FB'}, '+<',$self->{'FB_DEVICE'})) {
-        binmode($self->{'FB'});
+    if (open($self->{'FB'}, '+<',$self->{'FB_DEVICE'})) { # Can we open the framebuffer device??
+        # YAY!  We can, so let's set things up
 
+        binmode($self->{'FB'}); # We have to be in binary mode first
+
+        # Make the IOCTL call to get info on the virtual (viewable) screen (Sometimes different than physical)
         ($self->{'xres'}, $self->{'yres'}, $self->{'xres_virtual'}, $self->{'yres_virtual'}, $self->{'xoffset'}, $self->{'yoffset'}, $self->{'bits_per_pixel'}, $self->{'grayscale'}, $self->{'bitfields'}, $self->{'nonstd'}, $self->{'activate'}, $self->{'height'}, $self->{'width'}, $self->{'accel_flags'}, $self->{'pixclock'}, $self->{'left_margin'}, $self->{'right_margin'}, $self->{'upper_margin'}, $self->{'lower_margin'}, $self->{'hsync_len'}, $self->{'vsync_len'}, $self->{'sync'}, $self->{'vmode'}, @dummy) = _get_info($self->{'FBIOGET_VSCREENINFO'}, $self->{'FBioget_vscreeninfo'}, $self->{'FB'});
 
+        # MAke the IOCTL call to get info on the physical screen
         ($self->{'id'}, $self->{'smem_start'}, $self->{'smem_len'}, $self->{'type'}, $self->{'type_aux'}, $self->{'visual'}, $self->{'xpanstep'}, $self->{'ypanstep'}, $self->{'ywrapstep'}, $self->{'line_length'}, $self->{'mmio_start'}, $self->{'mmio_len'}, $self->{'accel'}, @dummy) = _get_info($self->{'FBIOGET_FSCREENINFO'}, $self->{'FBioget_fscreeninfo'}, $self->{'FB'});
+
         $self->{'VXRES'}          = $self->{'xres_virtual'};
         $self->{'VYRES'}          = $self->{'yres_virtual'};
         $self->{'XRES'}           = $self->{'xres'};
@@ -208,28 +239,33 @@ sub new {
         attribute_reset($self);
 
         # Now that everything is set up, let's map the framebuffer to SCREEN
+    
         mmap($self->{'SCREEN'}, $self->{'smem_len'}, PROT_READ | PROT_WRITE, MAP_SHARED, $self->{'FB'});
     } else {    # Go into emulation mode if no actual framebuffer available
+        # Aww bummer, no screen.  Ok, turn on emulation mode then
         $self->{'ERROR'}          = 'Framebuffer Device Not Found! Emulation mode.  EXPERIMENTAL!!';
+        
+        # Set the resolution.  Either the defaults, or whatever the user passed in.
         $self->{'SCREEN'}         = chr(0) x ($self->{'VXRES'} * $self->{'VYRES'} * $self->{'BYTES'});
-        $self->{'XRES'}           = $self->{'VXRES'};
+        $self->{'XRES'}           = $self->{'VXRES'}; # Virtual and physical are the same
         $self->{'YRES'}           = $self->{'VYRES'};
         $self->{'PIXELS'}         = (($self->{'XOFFSET'} + $self->{'VXRES'}) * ($self->{'YOFFSET'} + $self->{'VYRES'}));
         $self->{'SIZE'}           = $self->{'PIXELS'} * $self->{'BYTES'};
         $self->{'smem_len'}       = $self->{'BYTES'} * ($self->{'VXRES'} * $self->{'VYRES'}) if (!defined($self->{'smem_len'}) || $self->{'smem_len'} <= 0);
         $self->{'BYTES_PER_LINE'} = int($self->{'smem_len'} / $self->{'VYRES'});
+
         bless($self, $class);
 
         attribute_reset($self);
     } ## end else [ if (open($self->{'FB'}...))]
     return $self;
 } ## end sub new
-###############################################################
+##############################################################################
 
 =head2 screen_close
 
-Unmaps the SCREEN and closes the framebuffer.  This is usually
-automatically called when the object is destroyed.
+Unmaps the SCREEN and closes the framebuffer.  This is usually automatically
+called when the object is destroyed.
 
 =over 1
 
@@ -243,9 +279,9 @@ sub screen_close {
     if (!defined($self->{'ERROR'})) { # Only do it if not in emulation mode
         munmap($self->{'SCREEN'}) if (defined($self->{'SCREEN'}));
         close($self->{'FB'}) if (defined($self->{'FB'}));
-        delete($self->{'FB'});
+        delete($self->{'FB'}); # We leave no remnants MUHWAHAHAHAHA!!!
     }
-    delete($self->{'SCREEN'});
+    delete($self->{'SCREEN'}); # MUHWAHAHAHA!!
 }
 ###############################################################
 
@@ -412,9 +448,8 @@ sub cls {
 
 =head2 attribute_reset
 
-Resets the plot point at 0,0.  Resets clipping to the current
-screen size.  Resets the global color to white and resets the
-drawing mode to NORMAL.
+Resets the plot point at 0,0.  Resets clipping to the current screen size.
+Resets the global color to white and resets the drawing mode to NORMAL.
 
 =over 1
 
@@ -434,6 +469,22 @@ sub attribute_reset {
     $self->clip_reset;
 }
 
+=head2 reset
+
+The same as 'attribute_reset'.
+
+=over 1
+
+$fb->reset();
+
+=back
+=cut
+
+sub reset {
+    my $self = shift;
+    $self->attribute_reset();
+}
+
 =head2 plot
 
 Set a single pixel in the globally set color at position x,y
@@ -451,15 +502,14 @@ sub plot {
     my $self   = shift;
     my $params = shift;
 
-    my $x    = int($params->{'x'});                 # Ignore decimals
-    my $y    = int($params->{'y'});
+    my $x    = int($params->{'x'}) || 0;                 # Ignore decimals
+    my $y    = int($params->{'y'}) || 0;
     my $size = int($params->{'pixel_size'} || 1);
     my ($c, $index);
 
     if ($size > 1) {
         $self->circle({'x' => $x, 'y' => $y, 'radius' => ($size / 2), 'filled' => 1, 'pixel_size' => 1});
     } else {
-
         # Only plot if the pixel is within the clipping region
         if (   ($x <= $self->{'XX_CLIP'})
             && ($y <= $self->{'YY_CLIP'})
@@ -472,18 +522,12 @@ sub plot {
                 $c = substr($self->{'SCREEN'}, $index, $self->{'BYTES'});
                 switch ($self->{'DRAW_MODE'}) {
                     case ($self->{'XOR_MODE'}) {
-
-                        #                        $c = $c ^ $self->{'COLOR'};
                         $c ^= $self->{'COLOR'};
                     }
                     case ($self->{'OR_MODE'}) {
-
-                        #                        $c = $c | $self->{'COLOR'};
                         $c |= $self->{'COLOR'};
                     }
                     case ($self->{'AND_MODE'}) {
-
-                        #                        $c = $c & $self->{'COLOR'};
                         $c &= $self->{'COLOR'};
                     }
                     case ($self->{'MASK_MODE'}) {
@@ -528,13 +572,15 @@ sub drawto {
     my $self   = shift;
     my $params = shift;
 
-    my $x_end = int($params->{'x'});                 # Ignore decimals
-    my $y_end = int($params->{'y'});
+    my $x_end = int($params->{'x'}) || 0;    # Ignore decimals
+    my $y_end = int($params->{'y'}) || 0;
     my $size  = int($params->{'pixel_size'} || 1);
 
     my ($width, $height);
     my $start_x = $self->{'X'};
     my $start_y = $self->{'Y'};
+
+    # Determines if the coordinates sent were right-side-up or up-side-down.
     if ($start_x > $x_end) {
         $width = $start_x - $x_end;
     } else {
@@ -545,22 +591,25 @@ sub drawto {
     } else {
         $height = $y_end - $start_y;
     }
+    # We need only plot if start and end are the same
     if (($x_end == $start_x) && ($y_end == $start_y)) {
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
-    } elsif ($x_end == $start_x) {
-        if ($start_y > $y_end) {
+
+        # Else, let's get to drawing
+    } elsif ($x_end == $start_x) { # Draw a perfectly verticle line
+        if ($start_y > $y_end) { # Draw direction is UP
             while ($start_y >= $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_y--;
             }
-        } else {
+        } else { # Draw direction is DOWN
             while ($start_y <= $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_y++;
             }
         }
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
-    } elsif ($y_end == $start_y) {
+    } elsif ($y_end == $start_y) { # Draw a perfectly horizontal line (fast)
         if ($size == 1) {
             if ($start_x > $x_end) {
                 $self->blit_write({'x' => $x_end, 'y' => $y_end, 'width' => $width, 'height' => 1, 'image' => $self->{'COLOR'} x $width});    # Blitting a horizontal line is much faster!
@@ -577,27 +626,27 @@ sub drawto {
             }
         }
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
-    } elsif ($width > $height) {
+    } elsif ($width > $height) { # Wider than it is high
         my $factor = $height / $width;
-        if (($start_x < $x_end) && ($start_y < $y_end)) {
+        if (($start_x < $x_end) && ($start_y < $y_end)) { # Draw UP and to the RIGHT
             while ($start_x < $x_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_y += $factor;
                 $start_x++;
             }
-        } elsif (($start_x > $x_end) && ($start_y < $y_end)) {
+        } elsif (($start_x > $x_end) && ($start_y < $y_end)) { # Draw UP and to the LEFT
             while ($start_x > $x_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_y += $factor;
                 $start_x--;
             }
-        } elsif (($start_x < $x_end) && ($start_y > $y_end)) {
+        } elsif (($start_x < $x_end) && ($start_y > $y_end)) { # Draw DOWN and to the RIGHT
             while ($start_x < $x_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_y -= $factor;
                 $start_x++;
             }
-        } elsif (($start_x > $x_end) && ($start_y > $y_end)) {
+        } elsif (($start_x > $x_end) && ($start_y > $y_end)) { # Draw DOWN and to the LEFT
             while ($start_x > $x_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_y -= $factor;
@@ -605,27 +654,27 @@ sub drawto {
             }
         }
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
-    } elsif ($width < $height) {
+    } elsif ($width < $height) { # Higher than it is wide
         my $factor = $width / $height;
-        if (($start_x < $x_end) && ($start_y < $y_end)) {
+        if (($start_x < $x_end) && ($start_y < $y_end)) { # Draw UP and to the RIGHT
             while ($start_y < $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x += $factor;
                 $start_y++;
             }
-        } elsif (($start_x > $x_end) && ($start_y < $y_end)) {
+        } elsif (($start_x > $x_end) && ($start_y < $y_end)) { # Draw UP and to the LEFT
             while ($start_y < $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x -= $factor;
                 $start_y++;
             }
-        } elsif (($start_x < $x_end) && ($start_y > $y_end)) {
+        } elsif (($start_x < $x_end) && ($start_y > $y_end)) { # Draw DOWN and to the RIGHT
             while ($start_y > $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x += $factor;
                 $start_y--;
             }
-        } elsif (($start_x > $x_end) && ($start_y > $y_end)) {
+        } elsif (($start_x > $x_end) && ($start_y > $y_end)) { # Draw DOWN and to the LEFT
             while ($start_y > $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x -= $factor;
@@ -634,31 +683,32 @@ sub drawto {
         }
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
     } else {    # $width == $height
-        if (($start_x < $x_end) && ($start_y < $y_end)) {
+        if (($start_x < $x_end) && ($start_y < $y_end)) { # Draw UP and to the RIGHT
             while ($start_y < $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x++;
                 $start_y++;
             }
-        } elsif (($start_x > $x_end) && ($start_y < $y_end)) {
+        } elsif (($start_x > $x_end) && ($start_y < $y_end)) { # Draw UP and to the LEFT
             while ($start_y < $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x--;
                 $start_y++;
             }
-        } elsif (($start_x < $x_end) && ($start_y > $y_end)) {
+        } elsif (($start_x < $x_end) && ($start_y > $y_end)) { # Draw DOWN and to the RIGHT
             while ($start_y > $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x++;
                 $start_y--;
             }
-        } elsif (($start_x > $x_end) && ($start_y > $y_end)) {
+        } elsif (($start_x > $x_end) && ($start_y > $y_end)) { # Draw DOWN and to the LEFT
             while ($start_y > $y_end) {
                 $self->plot({'x' => $start_x, 'y' => $start_y, 'pixel_size' => $size});
                 $start_x--;
                 $start_y--;
             }
         }
+        # Plot the last pixel
         $self->plot({'x' => $x_end, 'y' => $y_end, 'pixel_size' => $size});
     } ## end else [ if (($x_end == $start_x...))]
 } ## end sub drawto
@@ -700,23 +750,24 @@ Draws an arc of a circle at point x,y.
 =cut
 
 sub draw_arc {
-
     # This isn't exactly the fastest routine out there,
     # hence the "granularity" parameter, but it is pretty
     # neat.
     my $self   = shift;
     my $params = shift;
 
-    my $x             = int($params->{'x'});
-    my $y             = int($params->{'y'});
-    my $radius        = int($params->{'radius'});
-    my $start_degrees = 0 + sprintf('%.03f', $params->{'start_degrees'});
-    my $end_degrees   = 0 + sprintf('%.03f', $params->{'end_degrees'});
-    my $granularity   = 0 + sprintf('%.03f', $params->{'granularity'});
-    my $mode          = int($params->{'mode'});
+    my $x             = int($params->{'x'})      || 0;
+    my $y             = int($params->{'y'})      || 0;
+    my $radius        = int($params->{'radius'}) || 1;
+
+    my $start_degrees = 0 + sprintf('%.03f', $params->{'start_degrees'}) || 0;
+    my $end_degrees   = 0 + sprintf('%.03f', $params->{'end_degrees'})   || 360;
+    my $granularity   = 0 + sprintf('%.03f', $params->{'granularity'})   || .1;
+
+    my $mode          = int($params->{'mode'})      || 0;
     my $size          = int($params->{'pixel_size'} || 1);
     $size = 1 if ($mode == 1);
-    my ($sx, $sy, $degrees, $ox, $oy);
+    my ($sx, $sy, $degrees, $ox, $oy) = (0,0,1,1,1);
 
     $degrees = $start_degrees;
     if ($start_degrees > $end_degrees) {
@@ -782,11 +833,11 @@ sub draw_arc {
 
 =head2 ellipse
 
-Draw an ellipse at center position x,y with XRadius,
-YRadius.  Either a filled out outline is drawn based
-on the value of $filled.  The optional factor value
-varies from the default 1 to change the look and
-nature of the output.  Clipping Applies.
+Draw an ellipse at center position x,y with XRadius, YRadius.  Either a filled
+out outline is drawn based on the value of $filled.  The optional factor value
+varies from the default 1 to change the look and nature of the output.
+
+Clipping Applies.
 
 =over 1
 
@@ -803,27 +854,26 @@ nature of the output.  Clipping Applies.
 =cut
 
 sub ellipse {
-
     # The routine even works properly for XOR mode when
     # filled ellipses are drawn as well.  This was solved by
     # drawing only if the X or Y position changed.
     my $self   = shift;
     my $params = shift;
 
-    my $cx      = int($params->{'x'});
-    my $cy      = int($params->{'y'});
-    my $XRadius = int($params->{'xradius'});
-    my $YRadius = int($params->{'yradius'});
+    my $cx      = int($params->{'x'}) || 0;
+    my $cy      = int($params->{'y'}) || 0;
+    my $XRadius = int($params->{'xradius'}) || 1;
+    my $YRadius = int($params->{'yradius'}) || 1;
     $XRadius    = 1 if ($XRadius < 1);
     $YRadius    = 1 if ($YRadius < 1);
 
     my $filled = int($params->{'filled'});
-    my $fact   = $params->{'factor'} || 0;
+    my $fact   = $params->{'factor'}     || 1;
     my $size   = $params->{'pixel_size'} || 1;
     $size      = 1 if ($filled);
 
-    my ($old_cyy, $old_cy_y);
-    if ($fact == 0) {
+    my ($old_cyy, $old_cy_y) = (0,0);
+    if ($fact == 0) { # We don't allow zero values for this
         $fact = 1;
     }
     my $TwoASquare   = (2 * ($XRadius * $XRadius)) * $fact;
@@ -945,16 +995,21 @@ sub circle {
 
 =head2 polygon
 
-Creates an empty polygon drawn in the global color value.  The
-parameter 'coordinates' is an array of x,y values.  The last
-x,y combination is connected automatically with the first to
-close the polygon.  All x,y values are absolute, not relative.
+Creates an empty polygon drawn in the global color value.  The parameter
+'coordinates' is a reference to an array of x,y values.  The last x,y
+combination is connected automatically with the first to close the polygon.
+All x,y values are absolute, not relative.
+
 Clipping applies.
 
 =over 1
 
  $fb->polygon({
-    'coordinates' => [5,5,23,34,7,7],
+    'coordinates' => [
+        5,5,
+        23,34,
+        7,7
+    ],
     'pixel_size'  => 4
  });
 
@@ -965,8 +1020,8 @@ sub polygon {
     my $self   = shift;
     my $params = shift;
 
-    my $size = int($params->{'pixel_size'} || 1);
-    my @coords = @{$params->{'coordinates'}};
+    my $size      = int($params->{'pixel_size'} || 1);
+    my @coords    = @{$params->{'coordinates'}};
     my ($xx, $yy) = (int(shift(@coords)), int(shift(@coords)));
     my ($x, $y);
     $self->plot({'x' => $xx, 'y' => $yy, 'pixel_size' => $size});
@@ -981,8 +1036,11 @@ sub polygon {
 
 =head2 box
 
-Draws a box from point x,y to point xx,yy, either as an outline,
-if 'filled' is 0, or as a filled block, if 'filled' is 1.
+Draws a box from point x,y to point xx,yy, either as an outline, if 'filled'
+is 0, or as a filled block, if 'filled' is 1.  Filled boxes draw faster than
+frames.
+
+Clipping applies.
 
 =over 1
 
@@ -1032,7 +1090,10 @@ sub box {
 
 Draws a box at point x,y with the width 'width' and height 'height'.
 It draws a frame if 'filled' is 0 or a filled box if 'filled' is 1.
-'pixel_size' only applies if 'filled' is 0.
+'pixel_size' only applies if 'filled' is 0.  Filled boxes draw
+faster than frames.
+
+Clipping applies.
 
 =over 1
 
@@ -1087,12 +1148,20 @@ sub set_color {
     my $G = int($params->{'green'}) & 255;
     my $B = int($params->{'blue'}) & 255;
     if ($self->{'BITS'} == 32) {
-        $self->{'COLOR'} = chr($B) . chr($G) . chr($R) . chr(255);
+        unless($self->{'COLOR_ORDER'}) { # BGR
+            $self->{'COLOR'} = chr($B) . chr($G) . chr($R) . chr(255);
+        } else { # RGB
+            $self->{'COLOR'} = chr($R) . chr($G) . chr($B) . chr(255);
+        }
     } else {
         $R = int($R / 8);
         $G = int($G / 8);
         $B = int($B / 8);
-        $self->{'COLOR'} = ($R << 11) + ($G << 6) + $B;
+        unless($self->{'COLOR_ORDER'}) { # BGR
+            $self->{'COLOR'} = ($B << 11) + ($G << 6) + $R;
+        } else { # RGB
+            $self->{'COLOR'} = ($R << 11) + ($G << 6) + $B;
+        }
         $self->{'COLOR'} = pack('S', $self->{'COLOR'});
     }
     $self->{'I_COLOR'} = Imager::Color->new($R, $G, $B);
@@ -1121,7 +1190,11 @@ sub set_b_color {
     my $G = int($params->{'green'}) & 255;
     my $B = int($params->{'blue'}) & 255;
     if ($self->{'BITS'} == 32) {
-        $self->{'B_COLOR'} = chr($B) . chr($G) . chr($R) . chr(255);
+        unless($self->{'COLOR_ORDER'}) { # BGR
+            $self->{'B_COLOR'} = chr($B) . chr($G) . chr($R) . chr(255);
+        } else {
+            $self->{'B_COLOR'} = chr($R) . chr($G) . chr($B) . chr(255);
+        }
     } else {
         $R = int($R / 8);
         $G = int($G / 8);
@@ -1131,6 +1204,18 @@ sub set_b_color {
     }
 } ## end sub set_b_color
 
+=head2 set_background_color
+
+Same as set_b_color
+
+=back
+=cut
+
+sub set_background_color {
+    my $self = shift;
+    $self->set_b_color(@_);
+}
+
 =head2 pixel
 
 Returns the color of the pixel at coordinate x,y.
@@ -1157,20 +1242,32 @@ sub pixel {
 
     my $x = int($params->{'x'});
     my $y = int($params->{'y'});
+
+    # Values outside of the clipping area return undefined.
     if (($x > $self->{'XX_CLIP'}) || ($y > $self->{'YY_CLIP'}) || ($x < $self->{'X_CLIP'}) || ($y < $self->{'Y_CLIP'})) {
-        return (undef);
+        return (undef,undef,undef,undef);
     } else {
         my ($color, $R, $G, $B, $A);
         my $index = ($self->{'BYTES_PER_LINE'} * ($y + $self->{'YOFFSET'})) + ($x * $self->{'BYTES'});
         $color = substr($self->{'SCREEN'}, $index, $self->{'BYTES'});
         if ($self->{'BITS'} == 32) {
-            ($B, $G, $R, $A) = unpack('C4', $color);
+            unless($self->{'COLOR_ORDER'}) { # BGR
+                ($B, $G, $R, $A) = unpack('C4', $color);
+            } else {
+                ($R, $G, $B, $A) = unpack('C4', $color);
+            }
         } else {
             $A = unpack('S', $color);
             $color = pack('S', $A);
-            $B     = $A & 31;
-            $G     = ($A >> 6) & 31;
-            $R     = ($A >> 11) & 31;
+            unless($self->{'COLOR_ORDER'}) { # BGR
+                $B     = $A & 31;
+                $G     = ($A >> 6) & 31;
+                $R     = ($A >> 11) & 31;
+            } else {
+                $R     = $A & 31;
+                $G     = ($A >> 6) & 31;
+                $B     = ($A >> 11) & 31;
+            }
             $R     = int($R * 8);
             $G     = int($G * 8);
             $B     = int($B * 8);
@@ -1185,6 +1282,7 @@ Does a flood fill starting at point x,y.  It samples the color
 at that point and determines that color to be the "background"
 color, and proceeds to fill in, with the current global color,
 until the "background" color is replaced with the new color.
+
 Clipping applies.
 
 BECAUSE OF ITS RECURSIVE NATURE, IT CAN CHOW DOWN ON MEMORY
@@ -1220,7 +1318,7 @@ sub _flood {
     ##########################################################
     ##                         FLOOD                        ##
     ##########################################################
-    # Used by FLOOD FILL above to flood file an empty space  #
+    # Used by FLOOD FILL above to flood fill an empty space  #
     # It starts at X,Y.  This can be a memory hog due to the #
     # recursive calls it makes.                              #
     ##########################################################
@@ -1270,14 +1368,7 @@ sub replace_color {
     my $new_r = int($params->{'new_red'});
     my $new_g = int($params->{'new_green'});
     my $new_b = int($params->{'new_blue'});
-    if ($self->{'BITS'} < 32) {
-        $old_r = int($old_r / 8);
-        $old_g = int($old_g / 8);
-        $old_b = int($old_b / 8);
-        $new_r = int($new_r / 8) * 8;
-        $new_g = int($new_g / 8) * 8;
-        $new_b = int($new_b / 8) * 8;
-    }
+
     $self->set_color({'red' => $new_r, 'green' => $new_g, 'blue' => $new_b});
     my $old_mode = $self->{'DRAW_MODE'};
     $self->{'DRAW_MODE'} = $self->{'NORMAL_MODE'};
@@ -1330,7 +1421,8 @@ sub blit_copy {
 =head2 blit_read
 
 Reads in a square portion of screen data at x,y,width,height,
-and returns the block of raw data as a string.
+and returns a hash pointer with information about the block,
+including the raw data as a string.
 
 =over 1
 
@@ -1341,6 +1433,20 @@ and returns the block of raw data as a string.
     'height' => 100
  });
 
+=back
+
+Returns:
+
+=over 1
+
+ {
+     'x'      => original X position,
+     'y'      => original Y position,
+     'width'  => width,
+     'height' => height,
+     'image'  => string of image data for the block
+ }
+
 =back
 =cut
 
@@ -1367,13 +1473,15 @@ sub blit_read {
         $scrn .= substr($self->{'SCREEN'}, $index, $W);
     }
 
-    return ({'width' => $w, 'height' => $h, 'image' => $scrn});
+    return ({'x' => $x, 'y' => $y, 'width' => $w, 'height' => $h, 'image' => $scrn});
 } ## end sub blit_read
 
 =head2 blit_write
 
 Writes a previously read block of screen data at x,y,width,height.
 
+It takes a hash reference.  It draws in the current drawing mode.
+
 =over 1
 
  $fb->blit_write({
@@ -1571,10 +1679,28 @@ Prints TrueType text on the screen at point x,y in the rectangle width,height,
 using the color 'color', and the face 'face'.
 
 This is best called twice, first in bounding box mode, and then in normal mode.
+
+Bounding box mode gets the actual values needed to display the text.
+
+=over 1
+
+ my $bounding_box = $fb->ttf_print({
+     'x'            => 20,
+     'y'            => 100,
+     'height'       => 16,
+     'color'        => 'FFFF00', # Yellow
+     'text'         => 'Hello World!',
+     'face'         => 'Arial',
+     'bounding_box' => 1,
+     'center'       => 0
+ });
+
+ $fb->ttf_print($bounding_box);
+
+=back
 =cut
 
 sub ttf_print {
-
     # Yes, this is a "hack".                                                     #
 
     # This uses the 'Imager' package.  It allocates a temporary screen buffer    #
@@ -1589,8 +1715,8 @@ sub ttf_print {
 
     my $TTF_x       = int($params->{'x'});
     my $TTF_y       = int($params->{'y'});
-    my $TTF_w       = int($params->{'width'});
-    my $TTF_h       = int($params->{'height'});
+    my $TTF_w       = int($params->{'width'})  || 1;
+    my $TTF_h       = int($params->{'height'}) || 1;
     my $P_color     = $params->{'color'};
     my $text        = $params->{'text'};
     my $face        = $params->{'face'};
@@ -1609,9 +1735,21 @@ sub ttf_print {
         );
 
         ($neg_width, $global_descent, $pos_width, $global_ascent, $descent, $ascent, $advance_width, $right_bearing) = $font->bounding_box('string' => $text, 'canon' => 1, 'size' => $TTF_h);
-        if ($box_mode == 1) {
-            return ({'width' => $advance_width, 'height' => ($global_ascent - $global_descent)});
-        } elsif ($center_mode == 1) {
+        if ($box_mode) {
+            return (
+                {
+                    'x'            => $params->{'x'},
+                    'y'            => $params->{'y'},
+                    'width'        => $advance_width,
+                    'height'       => ($global_ascent - $global_descent),
+                    'color'        => $params->{'color'},
+                    'text'         => $params->{'text'},
+                    'bounding_box' => 0,
+                    'face'         => $params->{'face'},
+                    'center'       => $params->{'center'}
+                }
+            );
+        } elsif ($center_mode) {
             $TTF_x = int(($self->{'XRES'} - $advance_width) / 2);
             $TTF_y = int((($self->{'YRES'} - $global_ascent) / 2) + $global_ascent);
         }
@@ -1641,7 +1779,14 @@ sub ttf_print {
         $self->blit_write({'x' => $TTF_x, 'y' => (($TTF_y - $TTF_h) + 1), 'width' => $TTF_w, 'height' => (($TTF_h + $global_ascent) - $global_descent), 'image' => $data});
     };
     print STDERR $@ if ($@);
-    return ({'x' => $TTF_x, 'y' => $TTF_y - $TTF_h, 'width' => $TTF_w, 'height' => ($TTF_h + $global_ascent) - $global_descent});
+    return (
+        {
+            'x'            => $TTF_x,
+            'y'            => $TTF_y - $TTF_h,
+            'width'        => $TTF_w,
+            'height'       => ($TTF_h + $global_ascent) - $global_descent
+        }
+    );
 } ## end sub ttf_print
 
 =head2 get_face_name
@@ -1895,6 +2040,39 @@ sub _set_info {
 
 __END__
 
+=head1 USAGE HINTS
+
+=head2 THREADS
+
+The module can NOT have separate threads calling the same object.
+You WILL crash. However, you can instantiate an object for each
+thread and it will work just fine!
+
+See the "examples" directory for "frtest.pl" as an example of a
+threading script that uses this module.  Just add the number of
+threads you want it to use to the command line when you run it.
+
+=head2 BLITTING
+
+Use blit_read and blit_write to save portions of the screen
+instead of redrawing everything.  It will speed up response
+tremendously.
+
+=head2 HORIZONTAL "MAGIC"
+
+Horizontal lines and filled boxes draw very fast.  Learn to
+exploit them.
+
+Filled boxes draw faster than thick horizontal lines.  See
+where I'm going?
+
+=head2 PIXEL SIZE
+
+Pixel sizes over 1 utilize a filled "circle" to do the drawing.
+This is why the larger the "pixel", the slower the draw.  Maybe
+for speed sake, a loop of lines drawn at single pixel size, but
+offset to make a "wide" line, will draw faster for you.  If making
+a game, or user interface, you may want to consider this technique.
 
 =head1 AUTHOR
 
@@ -1909,14 +2087,14 @@ modify it under the same terms as Perl itself.
 
 =head1 VERSION
 
-Version 4.10 (May 10, 2015)
+Version 4.11 (July 3, 2015)
 
 =head1 THANKS
 
 My thanks go out to those using this module and submitting helpful patches
 and suggestions for improvement:
 
+Troy Beisigl
 Markus Maier
 
 =cut
-
